<?php

namespace App\Service;

class Order {

    private $util;
    private $orderFactory;

    public function __construct(\App\Util\Util $util, 
                                \App\Factory\Order $orderFactory) {
        $this->util = $util;
        $this->orderFactory = $orderFactory;
    }

    public function history($params) {
        $uppercaseKeysArray = [];
        foreach ($params as $key => $value) {
            $uppercaseKey = strtoupper($key);
            $uppercaseKeysArray[$uppercaseKey] = $value;
        }

        $paramNames = ['DTINICIAL','DTFINAL', 'DTINIAGENDADA', 'DTFIMAGENDADA', 'CDFILIAL',
            'NRVENDAREST', 'NRCOMANDA', 'CLIENTECONSUMIDOR', 'IDSINCMSDE', 'IDRETBALLOJA',
            'IDORGCMDVENDA', 'IDSTCOMANDA', 'IDSINCAGENDA'];

        $cleanedParams = [];

        foreach ($paramNames as $paramName) {
            if (array_key_exists($paramName, $uppercaseKeysArray) && !empty($uppercaseKeysArray[$paramName])) {
                if (is_array($uppercaseKeysArray[$paramName])) {
                    $cleanedParams[$paramName] = implode(',', $uppercaseKeysArray[$paramName]);
                } else {
                    $cleanedParams[$paramName] = $uppercaseKeysArray[$paramName];
                }
            } else {
                $cleanedParams[$paramName] = 'T';
            }
        }
        $isScheduled = false;

        if (isset($params['dtIniAgendada']) && !empty($params['dtIniAgendada'])) {
            $cleanedParams['IDSTCOMANDA'] = 'A';
            $isScheduled = true;
        }

        return $this->orderFactory->history($cleanedParams, $isScheduled);
    }

    public function cancel($cdFilial, $cdOperador, $nrComanda, $nrVendaRest, $dsCancMotiv) {
        $cdVendedor = $this->orderFactory->getSeller($cdOperador);
        if (!$cdVendedor) {
            throw new \Exception("Operador não autorizado para cancelar pedidos.");
        }
        $orderStatus = $this->orderFactory->getOrderStatus($cdFilial, $nrVendaRest, $nrComanda);

        if (empty($orderStatus[0]['IDSTCOMANDA'])) {
            throw new \Exception("Pedido não encontrado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == "4") {
            throw new \Exception("Pedido já cancelado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == 'P' || $orderStatus[0]['IDSTCOMANDA'] == '5' || $orderStatus[0]['IDSTCOMANDA'] == '6') {
            throw new \Exception("Este pedido já foi impresso. Para cancelá-lo, utilize o Gestão de Vendas.");
        }

        $dsCancMotiv = $this->util->removeAcentos($dsCancMotiv);
        $dsCancMotiv = preg_replace('/[^A-Za-z0-9. -]/', '', $dsCancMotiv);
        $dsCancMotiv = strtoupper($dsCancMotiv);

        $cancelOrder = $this->orderFactory->cancelOrder($cdFilial, $nrVendaRest, $nrComanda, $cdOperador, $dsCancMotiv);
        $forceOrderSync = $this->orderFactory->forceOrderSync($cdFilial, $nrVendaRest);

        if ($cancelOrder->rowCount() === 0 || $forceOrderSync->rowCount() === 0) {
            throw new \Exception("Ocorreu um erro ao cancelar o pedido.");
        }

        return "Pedido cancelado com sucesso.";        
    }

    public function getBranchList()
    {

        $branchList = $this->orderFactory->getBranchList();

        return $branchList;
    }

    public function getOrderNumber($search)
    {

        $orderNumber = $this->orderFactory->getOrderNumber($search);

        return $orderNumber;
    }

    public function getConsumers($params)
    {
        $searchTerm = isset($params['consumers']) ? $params['consumers'] : '';
        return $this->orderFactory->getConsumers($searchTerm);
    }

    public function getPaymentMethods($params)
    {
        $paymentMethods = $this->orderFactory->getPaymentMethods($params);
        foreach ($paymentMethods as &$orderPayment) {
            if ($orderPayment['IDTIPOMOVIVEDLV'] == 'S') {
                $orderPayment['NMTIPORECE'] = 'TROCO';
            }
        }
        return $paymentMethods;
    }

    public function getOrderItemsRepo($params)
    {
        $products = array();
        $combos = $this->orderFactory->getOrderCombos($params);
        $items  = $this->orderFactory->getOrderItemsRepo($params);

        foreach ($items as &$item) {
            $item['ISCOMBO'] = false;
            $item['QTPRODCOMVEN'] = floatval($item['QTPRODCOMVEN']);
            $item['TOTAL'] = $this->util->floor($item['TOTAL']);
            array_push($products, $item);
        }

        foreach ($combos as $combo) {
            $combo['ISCOMBO'] = true;
            array_push($products, $combo);
            $comboIdxInProducts = count($products) - 1;
            $params = [
                'CDFILIAL'        => $combo['CDFILIAL'],
                'NRVENDAREST'     => $combo['NRVENDAREST'],
                'NRCOMANDA'       => $combo['NRCOMANDA'],
                'CDPRODPROMOCAO'  => $combo['CDPRODPROMOCAO'],
                'NRSEQPRODCOM'    => $combo['NRSEQPRODCOM'],
            ];
            $comboPrice = 0;
            $subItems = $this->orderFactory->getSubItens($params);
            foreach($subItems as $subItem) {
                $subItem['PARENTID'] = $combo['CDPRODUTO'];
                $subItem['TOTAL'] = floatval($subItem['TOTAL']);
                $products[] = $subItem;
                $comboPrice += $subItem['TOTAL'];
            }
            $products[$comboIdxInProducts]['TOTAL'] = $comboPrice;

        }

        return $products;
    }

    public function getItemObservation($params)
    {
        $explodedObs       = array();
        $observations      = $this->orderFactory->getItemObservation($params);
        $otherObservation  = $this->orderFactory->getOtherItemObservation($params);

        if ($otherObservation) {
            $explodedObs = explode(";", $otherObservation[0]['DSOCORR']);
            foreach ($explodedObs as $obsString) {
                $obsString = trim($obsString);
                $isRegularObservation = false;

                if (!empty($obsString)) {
                    foreach ($observations as $observation) {
                        if ($observation['DSOCORR'] == $obsString) {
                            $isRegularObservation = true;
                        }
                    }
                    if (!$isRegularObservation) {
                        $otherObservation['DSOCORR'] = $obsString;
                        array_push($observations, $otherObservation);
                    }
                }
            }
        }
        return $observations;
    }

    public function forwardOrder($cdFilial, $nrComanda, $nrVendaRest)
    {
        $orderStatus = $this->orderFactory->getOrderStatus($cdFilial, $nrVendaRest, $nrComanda);

        if (empty($orderStatus[0]['IDSTCOMANDA'])) {
            throw new \Exception("Pedido não encontrado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == "4") {
            throw new \Exception("Pedido já cancelado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == 'P' || $orderStatus[0]['IDSTCOMANDA'] == '5' || $orderStatus[0]['IDSTCOMANDA'] == '6') {
            throw new \Exception("Este pedido já foi impresso.");
        }

        $this->orderFactory->forwardOrderComandaven($cdFilial, $nrComanda, $nrVendaRest);
        $this->orderFactory->forwardOrderVendarest($cdFilial, $nrVendaRest);
        $this->orderFactory->forceOrderSync($cdFilial, $nrVendaRest);
    }

}
